***************************************************************
*** STATA v. 13 code to replicate the analysis of the paper ***
***      Grilli L., Rampichini C., Varriale R. (2015)       *** 
***  Statistical modelling of gained university credits     *** 
***  to evaluate the role of pre-enrolment assessment tests:***
***  an approach based on quantile regression for counts    ***
***************************************************************


*define working directory
cd [.....] /* put your directory address here */


* import data
import delimited "paper_grilli2015.csv"


* standardize test scores
egen std_Reading=std(reading)
egen std_Logic=std(logic)
egen std_Math=std(math)

label variable  std_Reading "Standardized reading score"
label variable  std_Logic "Standardized logic score"
label variable  std_Math "Standardized math score"


**************************************
*** Figure 2: Histogram of credits ***
**************************************
histogram credits, discrete width(1)  barwidth(0.1) percent ytitle(percent) fcolor(black)  lcolor(black) ///
          xtitle(Gained credits after one year) xlabel(0(3)60) scheme(s1mono) 



*******************************************************************
*** Multivariate linear regression model, Table 3 *****************
*******************************************************************
mvreg std_Logic std_Reading std_Math = female far_away hs_humanities hs_technical hs_other irreg hs_cgrade


*******************************************************************
*** Two-part model with quantile regression for counts, Table 4 ***
*******************************************************************
*generate the response variable ranging from 0 to 20
gen y=credits/3 

******************
* LOGISTIC MODEL *
******************

*generate binary response for Y>0
gen ypos=(y>0)

logit ypos i.female i.far_away i.hs_humanities i.hs_technical i.hs_other i.irreg hs_cgrade std_Logic std_Reading std_Math 

*** Table 4., column 1 ***

*predicted probability  for the baseline student
margins, at((base)_factor (zero) _continuous)  cformat(%6.3f) 

* conditional marginal effects for the baseline student
margins, dydx(*) at((base)_factor (zero) _continuous)   cformat(%6.3f) 

* conditional marginal effect of hs_cgrade for the baseline student
* from 0 (hs_grade=80) to 10 (hs_grade=90)
nlcom 1/(1+exp(-_b[_cons]-10*_b[hs_cgrade]))-1/(1+exp(-_b[_cons]))

* Wald test comparing this model with the null model
test 1.female 1.far_away 1.hs_humanities 1.hs_technical 1.hs_other 1.irreg hs_cgrade std_Logic std_Reading std_Math



**************************************
*** QUANTILE REGRESSION FOR COUNTS ***
**************************************
*to fit the quantile regression model you need the qcount command
*to install, type:

net install qcount.pkg

* The quantile of interest is declared in the q( ) option of the qcount command
* The following code is an example for quantile 0.10, thus use the option q(0.10)

* To fit the model for different quantiles change the value of q() 
*    and the value of the scalar tau in the post-estimation commands


***********
*** Q10 ***
***********
qcount y female far_away hs_humanities hs_technical hs_other irreg hs_cgrade std_Logic std_Reading std_Math if (y>0), q(0.10) rep(1000)
*the coef. in the output of qcount are the "average-jittering" estimates from eq. (3.4) in the paper
*The option rep() allows to choose the number of replicates: we set rep(1000), as in the paper
*Remark: due to random draws nedeed for jittering, the following results may be sligthly different from those reported in the paper

*save estimated coeff and their covariance matrix
estimate store model1
matrix b = e(b)
matrix v = e(V)


*post-estimation computations

*** Table 4., column 2: q(0.10) ***

*set the order of the quantile 
scalar tau=0.10    /* put here the same value as in q() */

*predicted quantile for 3*Z
*this is the value reported in Table 4
di 3*(tau+exp(_b[_cons]))         /* eq. (3.2) in the paper */

*predicted quantile for 3*Y (positive credits in the original scale, ranging from 3 to 60)
di ceil(3*(tau+exp(_b[_cons]))-1) /* eq. (3.3) in the paper */

*marginal effects on Q_z (see section 3.2) for the baseline student 
nlcom (Female:   3*(exp(_b[_cons]+_b[female])-exp(_b[_cons]))) (Far_away: 	3*(exp(_b[_cons]+_b[far_away])-exp(_b[_cons])))/*
	*/(HS_Human: 3*(exp(_b[_cons]+_b[hs_humanities])-exp(_b[_cons]))) (HS_Tech:	3*(exp(_b[_cons]+_b[hs_technical])-exp(_b[_cons])))/*
	*/(HS_Other: 3*(exp(_b[_cons]+_b[hs_other])-exp(_b[_cons])))	(HS_Irreg: 	3*(exp(_b[_cons]+_b[irreg])-exp(_b[_cons])))/*
	*/(HS_MaxGr: 3*(exp(_b[_cons]+10*_b[hs_cgrade])-exp(_b[_cons]))) (Logic: 3* exp(_b[_cons])*_b[std_Logic]) /*
	*/(Reading:  3*exp(_b[_cons])*_b[std_Reading]) 	(Math:	 	3* exp(_b[_cons])*_b[std_Math] 	), cformat(%6.3f) 	

	

*********************************************
*** Wald test: null model vs fitted model ***
*********************************************

*select coeff for H0: coeff=0 (the intercept is the last coeff in the parameter vector)
matrix p=b[1,1..10] /* select the first 10 elements, since the number of covariates in our application is 10 */
*select variances and covariances for the selected coeff
matrix cv=v[1..10,1..10]

*Wald statistic
matrix w=p*inv(cv)*p'
matrix list w, format(%6.1f)
*Right-tailed chi-squared distribution function
di  chi2tail(10,w[1,1])


********************************************
*** pseudo-R^2 of eq. (3.5) in the paper ***
********************************************
*compute V_full
gen yhat=3*ceil((tau+exp(_b[_cons]+_b[female]*female+_b[far_away]*far_away+_b[hs_humanities]*hs_humanities +_b[hs_technical]*hs_technical+_b[hs_other]*hs_other/*
         */+_b[irreg]*irreg+_b[hs_cgrade]*hs_cgrade +_b[std_Logic]*std_Logic+_b[std_Reading]*std_Reading +_b[std_Math]*std_Math))-1) if y>0

gen rasw=tau*abs(credits-yhat)*(credits >=yhat)+(1-tau)*abs(credits-yhat)*(credits<yhat) if y>0
quietly tabstat rasw, stat(sum) save
matrix rasw= r(StatTotal)

*fit the null model
qcount y if y>0, q(0.10) rep(1000)
matrix b0=e(b)

*compute V_null
gen y0hat=3*ceil((tau+exp(b0[1,1]))-1) if y>0
gen tasw=tau*abs(credits-y0hat)*(credits >=y0hat)+(1-tau)*abs(credits-y0hat)*(credits<y0hat) if y>0
quietly tabstat tasw, stat(sum) save
*return list
matrix tasw= r(StatTotal)


*pseudo r^2
matrix r=1-rasw[1,1]/tasw[1,1]
matrix list r, format(%6.3f) 
 
*** end file ***
